/*
 * main.c - topologyd main
 *
 * Copyright (C) 2020 iopsys Software Solutions AB. All rights reserved.
 *
 * See LICENSE file for license terms.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>

#include "debug.h"

extern int topologyd_start(void);
extern int topologyd_stop(void);

const char *ubus_socket = NULL;
const char *pidfile_path = NULL;
int refresh_int;
bool syslogging = false;
bool daemonize = false;

void do_daemonize()
{
	char buf[128] = {0};
	int flags;
	int f;

	if (daemonize) {
		info("daemonizing ...\n");
		daemon(0,0);
	}

	pidfile_path = "/var/run/topologyd.pid";
	f = open(pidfile_path, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
	if (f) {
		flags = fcntl(f, F_GETFD);
		if (flags != -1) {
			flags |= FD_CLOEXEC;
			fcntl(f, F_SETFD, flags);
		}
		if (lockf(f, F_TLOCK, 0) < 0) {
			fprintf(stderr, "File '%s' exists. Aborting...\n",
					pidfile_path);
			exit(-1);
		}
		ftruncate(f,0);
		snprintf(buf, sizeof(buf), "%ld\n", (long)getpid());
		write(f, buf, strlen(buf));
	}
}

void usage(char *prog)
{
    fprintf(stderr, "Usage: %s [options]\n", prog);
    fprintf(stderr, "\n");
    fprintf(stderr, "options:\n");
    fprintf(stderr, "   -s <socket path>     ubus socket\n");
    fprintf(stderr, "   -d,                  debug mode; i.e. don't daemonize\n");
    fprintf(stderr, "   -l,                  log to syslog\n");
    fprintf(stderr, "   -p <pidfile>         pid file path\n");
    fprintf(stderr, "   -t                   refresh interval in seconds\n");
    fprintf(stderr, "\n");
}

int main(int argc, char * argv[])
{
	int ch;

	while ((ch = getopt(argc, argv, "hlt:ds:p:")) != -1) {
		switch (ch) {
		case 's':
			ubus_socket = optarg;
			break;
		case 't':
			refresh_int = atoi(optarg);
			break;
		case 'l':
			syslogging = true;
			break;
		case 'd':
			daemonize = false;
			break;
		case 'p':
			pidfile_path = optarg;
			break;
		case 'h':
			usage(argv[0]);
			exit(0);
		default:
			break;
		}
	}

	if (daemonize)
		syslogging = true;

	do_daemonize();
	start_logging(argv[0]);

	topologyd_start();

	stop_logging();

	return 0;
}
