#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if_arp.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <libubox/list.h>
#include <libubox/uloop.h>
#include <libubus.h>

#include <easy/easy.h>
#include "debug.h"
#include "hlist.h"
#include "topologyd.h"


void node_destroy(struct node *n)
{
	free(n);
}

struct node *node_create(uint8_t *hwaddr)
{
	struct node *n;

	n = calloc(1, sizeof(struct node));
	if (!n) {
		warn("OOM: Node alloc failed!\n");
		return NULL;
	}

	memcpy(n->hwaddr, hwaddr, 6);

	return n;
}

struct node *node_lookup(struct hlist_head *table, uint8_t *hwaddr)
{
	int hidx = node_hash(hwaddr);
	struct node *n;

	if (hwaddr_is_zero(hwaddr))
		return NULL;

	hlist_for_each_entry(n, &table[hidx], hlist) {
		if (!memcmp(hwaddr, n->hwaddr, 6))
			return n;
	}

	return NULL;
}

int node_del(struct hlist_head *table, uint8_t *hwaddr)
{
	int hidx = node_hash(hwaddr);
	struct node *n;

	n = node_lookup(table, hwaddr);
	if (!n)
		return -1;

	hlist_del(&n->hlist, &table[hidx]);
	dbg("Node " MACFMT " removed from topology\n", MAC2STR(n->hwaddr));

	node_destroy(n);
	return 0;
}

struct node *node_add(struct hlist_head *table, uint8_t *hwaddr)
{
	struct node *n;

	n = node_lookup(table, hwaddr);
	if (n) {
		dbg("Node " MACFMT " already in topology\n", MAC2STR(hwaddr));
		return n;
	}

	n = node_create(hwaddr);
	if (n) {
		int hidx = node_hash(hwaddr);

		hlist_add_head(&n->hlist, &table[hidx]);
		dbg("Node " MACFMT " added to topology\n", MAC2STR(hwaddr));
	}

	return n;
}

void node_print_all(struct hlist_head *table)
{
	struct node *n;
	int i;

	dbg("Nodes:\n");
	for (i = 0; i < NODE_HTABLE_SIZE; i++) {
		if (hlist_empty(&table[i]))
			continue;

		hlist_for_each_entry(n, &table[i], hlist) {
			dbg("Node: " MACFMT "\n", MAC2STR(n->hwaddr));
		}
	}
}
